@@ -330,7 +330,7 @@ class RetrieverSkeleton(ConfigurableBase):
                      - name - parameter name
                      - type - a type function to compare the parameter value
                      against (i.e. str, int, bool)
-                     - default - optional default value.  If not preseent, the
+                     - default - optional default value.  If not present, the
                      parameter is required.
 
       __str__(self) - return a simple string representing the class instance.
@@ -433,15 +433,16 @@ class RetrieverSkeleton(ConfigurableBase):
                 or (isinstance(mailbox, (str, unicode)) and mailbox)), (
             'bad mailbox %s (%s)' % (mailbox, type(mailbox))
         )
-        if isinstance(mailbox, str):
-            mailbox = mailbox.decode('utf-8')
         filename = self.oldmail_filename
-        if mailbox is None:
-            # No mailbox (POP), use above with no extension
-            pass
-        else:
+        if mailbox is not None:
+            if isinstance(mailbox, str):
+                mailbox = mailbox.decode('utf-8')
+            mailbox = re.sub(STRIP_CHAR_RE, '.', mailbox)
+            mailbox = mailbox.encode('utf-8')
             # Use oldmail file per IMAP folder
-            filename += '-' + re.sub(STRIP_CHAR_RE, '.', mailbox)
+            filename += '-' + mailbox
+        # else:
+            # mailbox is None, is POP, just use filename
         return filename
 
     def oldmail_exists(self, mailbox):
@@ -706,7 +707,7 @@ class POP3RetrieverBase(RetrieverSkeleton):
             response, lines, octets = self.conn.retr(msgnum)
             self.log.debug('RETR response "%s", %d octets'
                            % (response, octets) + os.linesep)
-            msg = Message(fromlines=lines)
+            msg = Message(fromlines=lines+[''])
             return msg
         except poplib.error_proto, o:
             raise getmailRetrievalError(
@@ -1197,7 +1198,7 @@ class IMAPRetrieverBase(RetrieverSkeleton):
                 'Could not parse google imap extensions. Server said: %s'
                 % repr(response)
             )
-            return ()
+            return {}
 
         results = ext.groupdict()
         metadata = {}
