@@ -7,20 +7,37 @@ __all__ = [
     'Message',
 ]
 
+import sys
 import os
 import time
 import cStringIO
+import re
 import email
 import email.Errors
 import email.Utils
 import email.Parser
 from email.Generator import Generator
+try:
+    from email.header import Header
+except ImportError, o:
+    try:
+        from email.Header import Header
+    except ImportError, o:
+        # Python < 2.5
+        from email import Header
 
 from getmailcore.exceptions import *
 from getmailcore.utilities import mbox_from_escape, format_header, \
     address_no_brackets
 import getmailcore.logging
 
+if sys.hexversion < 0x02040000:
+    # email module in Python 2.3 uses more recursion to parse messages or
+    # similar; a user reported recursion errors with a message with ~300
+    # MIME parts.
+    # Hack around it by increasing the recursion limit.
+    sys.setrecursionlimit(2000)
+
 message_attributes = (
     'sender',
     'received_by',
@@ -29,6 +46,9 @@ message_attributes = (
     'recipient'
 )
 
+RE_FROMLINE = re.compile(r'^(>*From )', re.MULTILINE)
+
+
 #######################################
 def corrupt_message(why, fromlines=None, fromstring=None):
     log = getmailcore.logging.Logger()
@@ -130,19 +150,25 @@ class Message(object):
         it by writing out what we need, letting the generator write out the
         message, splitting it into lines, and joining them with the platform
         EOL.
+        
+        Note on mangle_from: the Python email.Generator class apparently only
+        quotes "From ", not ">From " (i.e. it uses mboxo format instead of
+        mboxrd).  So we don't use its mangling, and do it by hand instead.
         '''
-        f = cStringIO.StringIO()
         if include_from:
-            # This needs to be written out first, so we can't rely on the
-            # generator
-            f.write('From %s %s' % (mbox_from_escape(self.sender),
-                                    time.asctime()) + os.linesep)
+            # Mbox-style From line, not rfc822 From: header field.
+            fromline = 'From %s %s' % (mbox_from_escape(self.sender),
+                                       time.asctime()) + os.linesep
+        else:
+            fromline = ''
         # Write the Return-Path: header
-        f.write(format_header('Return-Path', '<%s>' % self.sender))
+        rpline = format_header('Return-Path', '<%s>' % self.sender)
         # Remove previous Return-Path: header fields.
         del self.__msg['Return-Path']
         if delivered_to:
-            f.write(format_header('Delivered-To', self.recipient or 'unknown'))
+            dtline = format_header('Delivered-To', self.recipient or 'unknown')
+        else:
+            dtline = ''
         if received:
             content = 'from %s by %s with %s' % (
                 self.received_from, self.received_by, self.received_with
@@ -151,13 +177,20 @@ class Message(object):
                 content += ' for <%s>' % self.recipient
             content += '; ' + time.strftime('%d %b %Y %H:%M:%S -0000',
                                             time.gmtime())
-            f.write(format_header('Received', content))
-        gen = Generator(f, mangle_from, 0)
+            receivedline = format_header('Received', content)
+        else:
+            receivedline = ''
         # From_ handled above, always tell the generator not to include it
         try:
+            tmpf = cStringIO.StringIO()
+            gen = Generator(tmpf, False, 0)
             gen.flatten(self.__msg, False)
-            f.seek(0)
-            return os.linesep.join(f.read().splitlines() + [''])
+            strmsg = tmpf.getvalue()
+            if mangle_from:
+                # do mboxrd-style "From " line quoting
+                strmsg = RE_FROMLINE.sub(r'>\1', strmsg)
+            return (fromline + rpline + dtline + receivedline 
+                    + os.linesep.join(strmsg.splitlines() + ['']))
         except TypeError, o:
             # email module chokes on some badly-misformatted messages, even
             # late during flatten().  Hope this is fixed in Python 2.4.
@@ -171,7 +204,7 @@ class Message(object):
                                 include_from)
 
     def add_header(self, name, content):
-        self.__msg[name] = content.rstrip()
+        self.__msg[name] = Header(content.rstrip())
 
     def remove_header(self, name):
         del self.__msg[name]
